/**
 * synopsis: Generate Xfce4 menu file from a JWM configuration file
 * purpose: The purpose of this utility is to keep the Xfce4 synchronized with the 
 *          JWM menu
 * usage: jwm2xfce-menu <.jwmrc> <menu.xml>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 */
#include <stdio.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#ifdef LIBXML_TREE_ENABLED

/*
 *To compile this file using gcc you can type
 *gcc `xml2-config --cflags --libs` -o jwm2xfce-menu jwm2xfce-menu.c
 */

//int processMenu = 0;
 
/**
 * process a node and all it's sibling and children nodes
 * @a_node: the initial xml node to consider.
 * @newParentNode: The current parent node in the new document.
 *
 * If the node belongs to the RootMenu element, it 
 * translates from the jwm menu nodes and attributes to the 
 * corresponding xfce menu nodes and attributes
 * and writes them to the destination document.
 */

void 
cleanup_input(char * jwmFileName)
{
    char *awkCommand1 = "awk '/<RootMenu/,/RootMenu>/' ";
    char *awkCommand2 = " | awk '/<!--/,/-->/ { next }; { print $0 }' > /tmp/jwmMenu";

    int  totLen = strlen(awkCommand1) + strlen(jwmFileName) + strlen(awkCommand2) + 1;
    
    char *awkCommand = (char *)calloc(totLen, sizeof(char));

    strcpy(awkCommand, awkCommand1);
    strcat(awkCommand, jwmFileName);
    strcat(awkCommand, awkCommand2);

    system(awkCommand);
    
    free(awkCommand);
    free(awkCommand1);
    free(awkCommand2);
}

static void
process_nodes(xmlNodePtr a_node, xmlNodePtr newParentNode)
{
  xmlNodePtr cur_node = NULL;
  xmlNodePtr newNode = NULL;

  for (cur_node = a_node; cur_node; cur_node = cur_node->next) 
  {
    if (cur_node->type == XML_ELEMENT_NODE) 
    {
        if (strcmp(cur_node->name, "Menu")==0)
        {
          xmlChar * name = xmlGetProp(cur_node, "label");
          xmlChar * icon = xmlGetProp(cur_node, "icon");
          newNode = xmlNewChild(newParentNode, NULL, BAD_CAST "menu", NULL);
          xmlNewProp(newNode, BAD_CAST "name", name);
          xmlNewProp(newNode, BAD_CAST "icon", icon);
        }
        else if (strcmp(cur_node->name, "Program")==0)
        {
          xmlChar * name = xmlGetProp(cur_node, "label");
          xmlChar * icon = xmlGetProp(cur_node, "icon");
          xmlChar * content = cur_node->children->content;
          if (strstr(content, "exec") != NULL)
          {
            xmlChar * content2 = malloc(strlen(content)-4);
            strcpy(content2, content+5);
            xmlFree(content);
            content = content2;
          }
          newNode = xmlNewChild(newParentNode, NULL, BAD_CAST "app", NULL);
          xmlNewProp(newNode, BAD_CAST "name", name);
          xmlNewProp(newNode, BAD_CAST "icon", icon);
          xmlNewProp(newNode, BAD_CAST "cmd", content);
        }
        else if (strcmp(cur_node->name, "Separator")==0)
        {
          newNode = xmlNewChild(newParentNode, NULL, BAD_CAST "separator", NULL);
        }
    }

    process_nodes(cur_node->children, newNode == NULL ? newParentNode : newNode);

  }
}


int
main(int argc, char **argv)
{
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;

    if (((argc < 2) || (argc > 3))
     || (strcmp(argv[1], "-h")==0)
     || (strcmp(argv[1], "-help")==0)
     || (strcmp(argv[1], "--help")==0))
    {
      printf ("Translator from a JWM configuration file to an Xfce4 menu file.\n");
      printf ("Usage:\n");
      printf ("  jwm2xfce-menu .jwmrc [menu.xml]\n");
      printf ("\n");
      printf ("  .jwmrc  : JWM configuration file\n");
      printf ("  menu.xml: Xfce4 menu file. If not provided, the output goes to the console\n\n");
      printf ("For example:\n");
      printf ("  jwm2xfce-menu \"/home/user/.jwmrc\" \"/home/user/.config/xfce4/desktop/menu.xml\"\n\n");
      return(1);
    }

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION
    
    // Build the awk command to get only the RootMenu section without comments
    cleanup_input(argv[1]);

    /*parse the file and get the DOM */
    doc = xmlReadFile("/tmp/jwmMenu", NULL, 0);

    if (doc == NULL) 
    {
        printf("error: could not parse file %s\n", argv[1]);
    }
    else
    {

      /* Initialize the destination document */
      xmlDocPtr newDoc = xmlNewDoc(BAD_CAST "1.0");
      xmlNodePtr newRootNode = xmlNewNode(NULL, BAD_CAST "xfdesktop-menu");
  
      xmlDocSetRootElement(newDoc, newRootNode);
      xmlDtdPtr dtd = xmlCreateIntSubset(newDoc, BAD_CAST "xfdesktop-menu", NULL, NULL);

      /*Get the root element node */
      root_element = xmlDocGetRootElement(doc);

      process_nodes(root_element, newRootNode);


      xmlSaveFormatFileEnc(argc > 2 ? argv[2] : "-", newDoc, "UTF-8", 1);

      /*free the document */
      xmlFreeDoc(newDoc);
    }

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    return 0;
}
#else
int main(void) {
    fprintf(stderr, "Tree support not compiled in\n");
    exit(1);
}
#endif
